@@ -414,9 +414,9 @@ class Tester:
         command += options.extra_options + " "
         command += fixed_opts + " "
         if fname is not None:
-            command += "--input %s " % fname
+            command += " %s " % fname
         if fname_frat:
-            command += " --frat %s " % fname_frat
+            command += " %s " % fname_frat
 
         if options.verbose:
             print("Executing before doalarm/pipes: %s " % command)
@@ -691,16 +691,14 @@ fuzzers_noxor = [
     ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
     ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
     ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
-    ["../../build/tests/cnf-utils/cnf-fuzz-biere"],
     ["../../build/tests/cnf-utils/makewff -cnf 3 250 1080", "-seed"],
     ["../../build/tests/cnf-utils/sgen4 -unsat -n 50", "-s"],
     ["../../build/tests/cnf-utils//sgen4 -sat -n 50", "-s"],
     ["../../utils/cnf-utils/cnf-fuzz-brummayer.py", "-s"],
     ["../../utils/cnf-utils/cnf-fuzz-xor.py", "--seed"],
+    ["../../utils/cnf-utils/cnf-fuzz-xor.py", "--seed"],
+    ["../../utils/cnf-utils/cnf-fuzz-xor.py", "--seed"],
+    ["../../utils/cnf-utils/cnf-fuzz-xor.py", "--seed"],
     ["../../utils/cnf-utils/multipart.py", "special"]
 ]
 
