@@ -24,6 +24,7 @@ THE SOFTWARE.
 
 #include <atomic>
 #include <limits>
+#include <random>
 
 #include "constants.h"
 #include "vardata.h"
@@ -45,6 +46,7 @@ THE SOFTWARE.
 #include <boost/serialization/vector.hpp>
 #endif
 
+
 namespace CMSat {
 
 class ClauseAllocator;
@@ -105,6 +107,7 @@ public:
         if (_conf != NULL) {
             conf = *_conf;
         }
+        mtrand.seed(conf.origSeed);
         frat = new Drat;
         assert(_must_interrupt_inter != NULL);
         must_interrupt_inter = _must_interrupt_inter;
@@ -119,6 +122,7 @@ public:
 
     ClauseAllocator cl_alloc;
     SolverConf conf;
+    std::mt19937_64 mtrand;
 
     bool ok = true; //If FALSE, state of CNF is UNSAT
 
