@@ -21,8 +21,8 @@ OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWA
 #ifndef Glucose_Heap_h
 #define Glucose_Heap_h
 
+#include <iostream>
 #include "Vec.h"
-#include "MersenneTwister.h"
 
 namespace CMSat {
 
@@ -118,12 +118,6 @@ public:
         assert(index < (int)heap.size());
         return heap[index];
     }
-    int random_element(MTRand& rnd)
-    {
-        assert(!heap.empty());
-        return heap[rnd.randInt(heap.size()-1)];
-    }
-
 
     void decrease  (int n)
     {
