@@ -20,13 +20,16 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 ***********************************************/
 
-#ifndef VARREPLACER_H
-#define VARREPLACER_H
+#pragma once
 
 #include <map>
 #include <vector>
 #include <utility>
 #include <tuple>
+#ifdef ARJUN_SERIALIZE
+#include <boost/serialization/vector.hpp>
+#include <boost/serialization/map.hpp>
+#endif
 
 #include "constants.h"
 #include "solvertypes.h"
@@ -73,6 +76,11 @@ class VarReplacer
         uint32_t get_var_replaced_with_outer(uint32_t var) const;
         bool var_is_replacing(const uint32_t var);
 
+#ifdef ARJUN_SERIALIZE
+        template<class T> void unserialize_tables(T& ar);
+        template<class T> void serialize_tables  (T& ar) const;
+#endif
+
         vector<uint32_t> get_vars_replacing(uint32_t var) const;
         void updateVars(
             const vector<uint32_t>& outerToInter
@@ -110,9 +118,6 @@ class VarReplacer
         bool get_scc_depth_warning_triggered() const;
         void delete_frat_cls();
 
-        void save_state(SimpleOutFile& f) const;
-        void load_state(SimpleInFile& f);
-
     private:
         Solver* solver;
         SCCFinder* scc_finder;
@@ -323,6 +328,20 @@ inline uint32_t VarReplacer::get_var_replaced_with_outer(uint32_t var) const
     return table[var].var();
 }
 
-} //end namespace
+#ifdef ARJUN_SERIALIZE
+template<class T>
+void VarReplacer::serialize_tables(T& ar) const
+{
+    ar << table;
+    ar << reverseTable;
+}
 
-#endif //VARREPLACER_H
+template<class T>
+void VarReplacer::unserialize_tables(T& ar)
+{
+    ar >> table;
+    ar >> reverseTable;
+}
+#endif
+
+} //end namespace
